
# q-expansion of newform 576.7.e.r, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 576
weight = 7
poly_data = [1446, 68, -67, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[30, -59, -3, 2], 153], [[-1704, 3392, 48, -32], 51], [[-408, -12, 12, 0], 1]]

hecke_ring_character_values = [[127, [1, 0, 0, 0]], [325, [1, 0, 0, 0]], [65, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, -25, 0, -1], [108, 0, 1, 0], [0, -524, 0, -6], [1088, 0, -7, 0], [0, -1633, 0, 2], [-6736, 0, -8, 0], [0, 9188, 0, -50], [0, 7761, 0, 67], [14044, 0, -131, 0], [59250, 0, -76, 0], [0, 34905, 0, 250], [712, 0, -514, 0], [0, 50292, 0, -298], [0, -127975, 0, 175], [0, -32504, 0, -708], [-36706, 0, 556, 0], [386552, 0, 302, 0], [0, 76300, 0, -1374], [70480, 0, -564, 0], [-314620, 0, 1067, 0], [0, 126668, 0, 686], [0, -275415, 0, -1716], [-330592, 0, 2350, 0], [0, 406215, 0, -531], [981044, 0, 3215, 0], [0, 176640, 0, 7552], [762464, 0, 4099, 0], [0, -705375, 0, 6764], [595340, 0, 1001, 0], [0, 2172920, 0, -4868], [0, -76857, 0, -4102], [-2184600, 0, -4934, 0], [0, -1199831, 0, -21683], [-3556004, 0, 2413, 0], [-3503998, 0, 5028, 0], [-684888, 0, -13314, 0], [0, -90600, 0, -10404], [0, -3386913, 0, 13939], [0, -836304, 0, -15376], [-1467744, 0, -8681, 0], [0, 2742696, 0, -25676], [-143218, 0, 17464, 0], [0, 2644407, 0, -27631], [-3596956, 0, -7125, 0], [-9835496, 0, -15250, 0], [-7748548, 0, 31757, 0], [0, 4087380, 0, -47646], [8320736, 0, -51107, 0], [0, -5950505, 0, 24508], [0, 12659200, 0, 18968], [-9143856, 0, 39238, 0], [0, -3239140, 0, 31518], [0, 3958175, 0, -33848], [0, -2758360, 0, -5700], [0, 16198639, 0, 21449], [-13162580, 0, 32513, 0], [-5542432, 0, -37005, 0], [0, 15093401, 0, 6136], [-1376752, 0, 36700, 0], [0, -12843927, 0, 1111], [-13156152, 0, -50998, 0], [0, -10911144, 0, 130788], [14890818, 0, 132264, 0], [0, -1956705, 0, -103771], [-1908760, 0, -116806, 0], [28700208, 0, -7744, 0], [0, 9378132, 0, -112958], [46763070, 0, 19692, 0], [0, -15868207, 0, -233780], [0, 26647620, 0, -122202], [-4705972, 0, -185047, 0], [2531758, 0, 83004, 0], [-62933056, 0, -52116, 0], [0, 7348320, 0, 276264], [0, 6520265, 0, 84101], [-20678018, 0, -271524, 0], [0, 2448735, 0, 314262], [-40419456, 0, 205618, 0], [0, -56941620, 0, -13090], [52466560, 0, 72075, 0], [0, -38860604, 0, 260350], [81172318, 0, -14208, 0], [-17217684, 0, 270609, 0], [0, -42089860, 0, -157178], [0, 9433585, 0, -487770], [-67936096, 0, 17750, 0], [0, -13914175, 0, 40593], [-66291980, 0, -74601, 0], [0, 3934692, 0, 437098], [0, 14649796, 0, -411050], [28861412, 0, 383651, 0], [0, -11814240, 0, 857368], [-87412704, 0, 45992, 0], [0, -129508620, 0, 219598], [0, 123394161, 0, -107985], [0, 60190391, 0, -75186], [36617152, 0, -300044, 0], [-101725056, 0, -178917, 0], [100783496, 0, -44450, 0], [0, 15820913, 0, -1006779], [0, -71581148, 0, 824306], [0, -186676169, 0, 248334], [-100360944, 0, -643532, 0], [-1930318, 0, -1146296, 0], [0, -76352360, 0, -398940], [0, 83209375, 0, 947052], [0, -25647900, 0, -1018082], [-82042754, 0, -706664, 0], [179542780, 0, -662907, 0], [-223645970, 0, -580196, 0], [0, -6901815, 0, -1631656], [-110235584, 0, -220968, 0], [-142315036, 0, -798645, 0], [0, -83695745, 0, 868674], [345632520, 0, 40950, 0], [0, -213204980, 0, -383398], [0, 22016127, 0, 1166175], [0, -60928440, 0, 334572], [-124276910, 0, -362108, 0], [291091810, 0, -624464, 0], [0, -142798393, 0, -1514717], [0, 147498220, 0, -418402], [295196552, 0, -91770, 0], [0, -37568319, 0, 1684183], [630528064, 0, -245407, 0], [0, 274145516, 0, 550082], [-302781612, 0, -757209, 0], [-60636800, 0, 1427185, 0], [-396343120, 0, 758940, 0], [0, 159461720, 0, 654652], [156896940, 0, 587305, 0], [201359968, 0, -323503, 0], [0, 90625145, 0, -337314], [-381134926, 0, -1044600, 0], [0, -303670215, 0, -406253], [-573203408, 0, -265552, 0], [0, 39952895, 0, -2088065], [0, 24309785, 0, -1033514], [-299189696, 0, -126276, 0], [0, -185935881, 0, 891145], [-762109892, 0, 390213, 0], [0, 199246520, 0, -2354596], [519431648, 0, 354715, 0], [0, 372031764, 0, 2210398], [-132631598, 0, 19812, 0], [0, 204517545, 0, 411402], [-592345272, 0, -854450, 0], [0, 123805880, 0, -5117572], [-564163938, 0, 1043372, 0], [0, -444928465, 0, 1838312], [315438008, 0, -1624414, 0], [0, -298148408, 0, -3998804], [44959240, 0, -1465602, 0], [0, -780236776, 0, 1267788], [-71200276, 0, -2275983, 0], [0, -501951121, 0, -3764634], [128538798, 0, -8400, 0], [0, 864208929, 0, 35681], [0, 631236648, 0, -1910804], [0, 437005575, 0, 3902454], [-351779660, 0, 1088663, 0], [0, 202317084, 0, -3030730], [0, -410416383, 0, -810782], [0, 463722360, 0, 2728692], [-734691604, 0, 541289, 0], [803410670, 0, 2266668, 0]]
